/* ------------------- Ratón basdo en Joystick v0.5 ---------------------
 *  
 * Ratón basado en un joystick arcade.
 *   Se definen 4 entradas para controlar los 4 ejes de movimento
 *   Se definen 4 entradas para definir 4 botones de gesión:
 *     - Botón izquierdo de ratón
 *     - Botón derecho de ratón
 *     - Botón simulación doble click. Con sólo presionar una vez el botón se 
 *       transmite un doble click al computador.
 *     - Botón de arastre. Para facilitar al usuario la función de arrastre
 *       con el botón izquierdo se incluye este botón. Al presionarlo la primera
 *       vez se simula que el botón izquierdo se mantiene presionado.
 *       Al presionarlo de nuevo se libera el botón izquierdo.
 *       
 *   Se definen además dos leds adiconales:
 *     - Led de ratón conectado. Se enciende al conectar el ratón
 *     - Led de arastre. Se mantiene encendido cuando la función de arrastre está
 *       activa.
 *       
 * 
 * Author: Asociación MóstolesMakers
 *         -Instagram: @mostolesmakers 
 *       
 * Fecha: 28-02-2022
 * ----------------------------------------------------------------------
 * Changelog:
 *    19-10-2022:
 *      Potenciómetro de control de velocidad eliminado
 */



#include <Mouse.h>

#define NUM_MOVS (4)
#define NUM_BUTTONS (4)

#define UP (2)
#define DOWN (3)
#define LEFT (0)
#define RIGHT (1)

#define LEFT_CLK (2)
#define LEFT_DB_CLK (3)
#define RIGHT_CLK (1)
#define LEFT_CLK_MAIN (0)

#define K_UP (1.07)
#define K_DOWN (1.07)
#define K_LEFT (1.07)
#define K_RIGHT (1.07)

#define MAX_UP (-15)
#define MAX_DOWN (15)
#define MAX_LEFT (-15)
#define MAX_RIGHT (15)

#define BASE_CHANGE (2)

#define TIME_LOOP (100)

#define ON_PIN (2)
#define SELECT_PIN (3)

// Pins connected to Rows (Y in Hardware):
int MovePins[NUM_MOVS] = {6, 7, 8, 9};
// Pins connected to Columns (X in Hardware):
int ButtonPins[NUM_BUTTONS] = {10, 14, 15, 16}; 

float xPos=0;
float yPos=0;

float upChange=1;
float downChange=1;
float leftChange=1;
float rightChange=1;

// Pins connected to Columns (X in Hardware):
bool ButtonPress[NUM_BUTTONS] = {false, false, false, false}; 

bool xChange = false;
bool yChange = false;
bool clkChange = false;
bool maintSel = false;

void setup() {
  Serial.begin(9600);
  Mouse.begin();
  resetPins();
  xChange = false; // By default Mouse not moved
  yChange = false; // By default Mouse not moved
  
  // Power On Pin
  pinMode(ON_PIN , OUTPUT); 
  digitalWrite(ON_PIN , HIGH);
  
  // Select pin
  pinMode(SELECT_PIN , OUTPUT);  
  
}
 
void loop() {
  resetPins();
  getKeyStatus();
  if (  xChange || yChange) {
    Mouse.move(xPos, yPos, 0);
    yPos = 0;
    xPos = 0;
  }
  delay(TIME_LOOP);
}

void getKeyStatus()
{
  if (!digitalRead(MovePins[UP]))
  {
        Serial.println("Up");
    yChange = true;
    upChange = upChange * K_UP;
    yPos = -1*upChange;
    if (yPos <= MAX_UP) {
      yPos = MAX_UP;
    }
  } else if (!digitalRead(MovePins[DOWN]))
  {
        Serial.println("Down");
    yChange = true;
    downChange = downChange * K_DOWN;
    yPos = downChange;
    if (yPos >= MAX_DOWN) {
      yPos = MAX_DOWN;
    }
  } else {
    downChange = BASE_CHANGE;
    upChange = BASE_CHANGE;
    yChange = false;
    
  }

  if (!digitalRead(MovePins[LEFT]))
  {
        Serial.println("Left");
    xChange = true;
    leftChange = leftChange * K_LEFT;
    xPos = -1* leftChange;
    if (xPos <= MAX_LEFT) {
      xPos = MAX_LEFT;
    }
  } else if (!digitalRead(MovePins[RIGHT]))
  {
        Serial.println("Right");
    xChange = true;
    rightChange = rightChange * K_RIGHT;
    xPos = rightChange;
    if (xPos >= MAX_RIGHT) {
      xPos = MAX_RIGHT;
    }
  } else {
    rightChange = BASE_CHANGE;
    leftChange = BASE_CHANGE;
    xChange = false;
  }
  if (xChange || yChange) {
        Serial.print("Moved changed: ");
        Serial.print(xPos);
        Serial.print(" - ");
        Serial.print(yPos);
        Serial.print("\n");
  }

  if (!digitalRead(ButtonPins[LEFT_CLK]))
  {
    if (Mouse.isPressed(MOUSE_RIGHT)) {
      Mouse.release(MOUSE_RIGHT);
    }
    clkChange = true;
    if (!Mouse.isPressed(MOUSE_LEFT)) {
      Serial.println("Left Click");
      Mouse.press(MOUSE_LEFT);
    }

  } else   if (!digitalRead(ButtonPins[LEFT_DB_CLK]) )
  {
    Serial.println("Double Click. TODO");
    if (!clkChange) {
      if (Mouse.isPressed(MOUSE_RIGHT)) {
        Mouse.release(MOUSE_RIGHT);
      }
      if (Mouse.isPressed(MOUSE_LEFT)) {
        Mouse.release(MOUSE_LEFT);
      }
      clkChange = true;
      if (!Mouse.isPressed(MOUSE_LEFT)) {
        Mouse.press(MOUSE_LEFT);
        delay(TIME_LOOP);
        Mouse.release(MOUSE_LEFT);
        delay(TIME_LOOP);
        Mouse.press(MOUSE_LEFT);
        delay(TIME_LOOP);
        Mouse.release(MOUSE_LEFT);
      }
    }
  } 
  else if (!digitalRead(ButtonPins[LEFT_CLK_MAIN]))
  {
    if (!clkChange) {

      clkChange = true;
      if (maintSel and Mouse.isPressed(MOUSE_LEFT)) {
          Serial.println("Release Maintain Click. TODO");
          Mouse.release(MOUSE_LEFT);
          digitalWrite(SELECT_PIN , LOW);
      }
      else if (!maintSel and !Mouse.isPressed(MOUSE_LEFT)) {
        Serial.println("Press Maintain Click. TODO");
        Mouse.press(MOUSE_LEFT);
        digitalWrite(SELECT_PIN , HIGH);
      }
      maintSel = !maintSel;
      Serial.println(maintSel);
    }
  }
  else if (!digitalRead(ButtonPins[RIGHT_CLK]))
  {
    
    clkChange = true;
    if (Mouse.isPressed(MOUSE_LEFT)) {
      Mouse.release(MOUSE_LEFT);
    }
    if (!Mouse.isPressed(MOUSE_RIGHT)) {
      Serial.println("Right Click");
      Mouse.press(MOUSE_RIGHT);
    }

  } else {
    if (Mouse.isPressed(MOUSE_RIGHT)) {
      Mouse.release(MOUSE_RIGHT);
    }
    if (!maintSel and Mouse.isPressed(MOUSE_LEFT)) {
      Mouse.release(MOUSE_LEFT);
    }
    clkChange = false;
  }

  resetPins();
  
  
}

void resetPins()
{
 
  /* initialize all pins, inputs w/ pull-ups */
  for (int i=0; i<NUM_MOVS; i++)
  {
    pinMode(MovePins[i], INPUT);
    digitalWrite(MovePins[i], HIGH);
  }
  for (int i=0; i<NUM_BUTTONS; i++)
  {
    pinMode(ButtonPins[i], INPUT);
    digitalWrite(ButtonPins[i], HIGH);
  }
}
